// Copyright Epic Games, Inc. All Rights Reserved.

#include "EndlessVendettaCharacter.h"
#include "Animation/AnimInstance.h"
#include "Camera/CameraComponent.h"
#include "Components/CapsuleComponent.h"
#include "EnhancedInputComponent.h"
#include "EnhancedInputSubsystems.h"
#include "AI/EnemyCharacter.h"
#include "GameFramework/CharacterMovementComponent.h"
#include "Kismet/GameplayStatics.h"


//////////////////////////////////////////////////////////////////////////
// AEndlessVendettaCharacter

AEndlessVendettaCharacter::AEndlessVendettaCharacter()
{
	// Character doesnt have a rifle at start
	bHasRifle = false;

	//Enable Crouching
	GetCharacterMovement()->GetNavAgentPropertiesRef().bCanCrouch = true;
	// Set size for collision capsule
	GetCapsuleComponent()->InitCapsuleSize(55.f, 96.0f);

	// Create a CameraComponent	
	FirstPersonCameraComponent = CreateDefaultSubobject<UCameraComponent>(TEXT("FirstPersonCamera"));
	FirstPersonCameraComponent->SetupAttachment(GetCapsuleComponent());
	FirstPersonCameraComponent->SetRelativeLocation(FVector(-10.f, 0.f, 60.f)); // Position the camera
	FirstPersonCameraComponent->bUsePawnControlRotation = true;

	// Create a mesh component that will be used when being viewed from a '1st person' view (when controlling this pawn)
	Mesh1P = CreateDefaultSubobject<USkeletalMeshComponent>(TEXT("CharacterMesh1P"));
	Mesh1P->SetOnlyOwnerSee(true);
	Mesh1P->SetupAttachment(FirstPersonCameraComponent);
	Mesh1P->bCastDynamicShadow = false;
	Mesh1P->CastShadow = false;
	//Mesh1P->SetRelativeRotation(FRotator(0.9f, -19.19f, 5.2f));
	Mesh1P->SetRelativeLocation(FVector(-30.f, 0.f, -150.f));
}

void AEndlessVendettaCharacter::BeginPlay()
{
	// Call the base class  
	Super::BeginPlay();

	//Add Input Mapping Context
	if (APlayerController* PlayerController = Cast<APlayerController>(Controller))
	{
		if (UEnhancedInputLocalPlayerSubsystem* Subsystem = ULocalPlayer::GetSubsystem<UEnhancedInputLocalPlayerSubsystem>(PlayerController->GetLocalPlayer()))
		{
			Subsystem->AddMappingContext(DefaultMappingContext, 0);
		}
	}
	AActor* GadgetManagerActor = GetWorld()->SpawnActor<AActor>(GadgetManagerClass, GetActorLocation(), GetActorRotation());
	GadgetManager = Cast<AGadgetManager>(GadgetManagerActor);
	FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);
	GadgetManagerActor->AttachToComponent(GetRootComponent(), AttachmentRules);
	for (UActorComponent* PlayersCamera : GetComponentsByTag(UCameraComponent::StaticClass(), FName("PlayersCamera")))
	{
		UE_LOG(LogTemp, Warning, TEXT("An actor component with PlayersCamera tag has been found"));
		GadgetManager->SpawnGadgets(Cast<USceneComponent>(PlayersCamera));
		break;
	}
}

//////////////////////////////////////////////////////////////////////////// Input

void AEndlessVendettaCharacter::SetupPlayerInputComponent(class UInputComponent* PlayerInputComponent)
{
	// Set up action bindings
	if (UEnhancedInputComponent* EnhancedInputComponent = CastChecked<UEnhancedInputComponent>(PlayerInputComponent))
	{
		//Jumping
		EnhancedInputComponent->BindAction(JumpAction, ETriggerEvent::Triggered, this, &ACharacter::Jump);
		EnhancedInputComponent->BindAction(JumpAction, ETriggerEvent::Completed, this, &ACharacter::StopJumping);

		//Moving
		EnhancedInputComponent->BindAction(MoveAction, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::Move);

		//Looking
		EnhancedInputComponent->BindAction(LookAction, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::Look);

		// Gadget Toggling
		EnhancedInputComponent->BindAction(ReconAction, ETriggerEvent::Started, this, &AEndlessVendettaCharacter::ToggleRecon);
		EnhancedInputComponent->BindAction(CombatAction, ETriggerEvent::Started, this, &AEndlessVendettaCharacter::ToggleCombat);

		//Weapon Switching
		EnhancedInputComponent->BindAction(EquipPrimaryWeapon, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::EquipPrimary);
		EnhancedInputComponent->BindAction(EquipSecondaryWeapon, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::EquipSecondary);

		//Weapon Shooting
		EnhancedInputComponent->BindAction(TapShootAction, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::FireCaller);
		EnhancedInputComponent->BindAction(TapShootAction, ETriggerEvent::Completed, this, &AEndlessVendettaCharacter::StopFire);
		EnhancedInputComponent->BindAction(GunAimInAction, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::GunRightClick);
		EnhancedInputComponent->BindAction(GunAimInAction, ETriggerEvent::Completed, this, &AEndlessVendettaCharacter::StopGunRightClick);

		//Weapon Reloading
		EnhancedInputComponent->BindAction(GunReloadAction, ETriggerEvent::Triggered, this, &AEndlessVendettaCharacter::GunReload);

		//Crouching
		EnhancedInputComponent->BindAction(CrouchAction, ETriggerEvent::Started, this, &AEndlessVendettaCharacter::SetCrouch);
		EnhancedInputComponent->BindAction(CrouchAction, ETriggerEvent::Completed, this, &AEndlessVendettaCharacter::SetUnCrouch);
	}
}

void AEndlessVendettaCharacter::SetCrouch()
{
	Crouch();
}

void AEndlessVendettaCharacter::SetUnCrouch()
{
	UnCrouch();
}

float AEndlessVendettaCharacter::TakeDamage(const float DamageAmount, FDamageEvent const& DamageEvent, AController* EventInstigator, AActor* DamageCauser)
{
	CurrentHealth -= DamageAmount;
	if (CurrentHealth <= 0)
	{
		CurrentHealth = 0;
		UE_LOG(LogTemp, Warning, TEXT("Player is dead"));
	}

	return Super::TakeDamage(DamageAmount, DamageEvent, EventInstigator, DamageCauser);
}

void AEndlessVendettaCharacter::ToggleRecon()
{
	if (IsValid(PrimaryWeapon)) EquipPrimary();
	if (IsValid(SecondaryWeapon)) EquipSecondary();

	if (GadgetManager->IsReconEquipped())
	{
		GadgetManager->TryToUnequipRecon();
		return;
	}

	if (GadgetManager->IsCombatEquipped() && !GadgetManager->TryToUnequipCombat())
	{
		// Do nothing if combat is equipped and can't be unequipped at this moment
		return;
	}

	GadgetManager->EquipRecon();
}

void AEndlessVendettaCharacter::ToggleCombat()
{
	if (IsValid(PrimaryWeapon)) EquipPrimary();
	if (IsValid(SecondaryWeapon)) EquipSecondary();

	if (GadgetManager->IsCombatEquipped())
	{
		GadgetManager->TryToUnequipCombat();
		return;
	}

	if (GadgetManager->IsReconEquipped() && !GadgetManager->TryToUnequipRecon())
	{
		// Do nothing if recon is equipped and can't be unequipped at the moment
		return;
	}

	GadgetManager->EquipCombat();
}


//When 1 is pressed it calls EquipPrimary
void AEndlessVendettaCharacter::EquipPrimary()
{
	if (IsValid(PrimaryWeapon))
	{
		PrimaryWeapon->Destroy();
		this->GetFirstPersonCameraComponent()->SetFieldOfView(90);
		PrimaryWeapon = nullptr;
		bHasRifle = false;
		return;
	}

	if (IsValid(SecondaryWeapon)) EquipSecondary();

	if (GadgetManager->IsReconEquipped() && !GadgetManager->TryToUnequipRecon()) return;
	if (GadgetManager->IsCombatEquipped() && !GadgetManager->TryToUnequipCombat()) return;


	FActorSpawnParameters spawnParams;
	spawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
	FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);

	bHasRifle = true;
	//Creating a new actor object called PrimaryWeapon that is based off primaryweaponClass
	AActor* PrimaryWeaponActor = GetWorld()->SpawnActor<AActor>(PrimaryWeaponClass, spawnParams);
	PrimaryWeaponActor->AttachToComponent(Mesh1P, AttachmentRules, FName("GripPoint"));
	//Changing PrimaryWeaponActor to ABaseWeaponClass type instead of actor and storing it into PrimaryWeapon which is a ABaseClass Object
	//We do this because we need to check if PrimaryWeapon is equipped and we want primaryweapon to be ABaseWeapon type and not a generic AActor
	PrimaryWeapon = Cast<ABaseWeaponClass>(PrimaryWeaponActor);

}

void AEndlessVendettaCharacter::EquipSecondary()
{
	if (IsValid(SecondaryWeapon))
	{
		SecondaryWeapon->Destroy();
		this->GetFirstPersonCameraComponent()->SetFieldOfView(90);
		SecondaryWeapon = nullptr;
		bHasRifle = false;
		return;
	}

	if (IsValid(PrimaryWeapon)) EquipPrimary();

	if (GadgetManager->IsReconEquipped() && !GadgetManager->TryToUnequipRecon()) return;
	if (GadgetManager->IsCombatEquipped() && !GadgetManager->TryToUnequipCombat()) return;


	FActorSpawnParameters spawnParams;
	spawnParams.SpawnCollisionHandlingOverride = ESpawnActorCollisionHandlingMethod::AlwaysSpawn;
	FAttachmentTransformRules AttachmentRules(EAttachmentRule::SnapToTarget, true);

	bHasRifle = true;
	AActor* SecondaryWeaponActor = GetWorld()->SpawnActor<AActor>(SecondaryWeaponClass, spawnParams);
	SecondaryWeaponActor->AttachToComponent(Mesh1P, AttachmentRules, FName("GripPoint"));
	SecondaryWeapon = Cast<ABaseWeaponClass>(SecondaryWeaponActor);
}

//Calls the fire function in the baseWeaponClass
void AEndlessVendettaCharacter::FireCaller()
{
	 if (IsValid(PrimaryWeapon))
	 {
		 PrimaryWeapon->Fire();
	 }
	 if (IsValid(SecondaryWeapon))
	 {
		 SecondaryWeapon->Fire();
	 }
}

void AEndlessVendettaCharacter::StopFire()
{
	if (IsValid(PrimaryWeapon))
	{
		PrimaryWeapon->CancelFire();
	}
	if (IsValid(SecondaryWeapon))
	{
		SecondaryWeapon->CancelFire();
	}
}

void AEndlessVendettaCharacter::GunRightClick()
{
	if (IsValid(PrimaryWeapon) && !bIsScoped)
	{
		for (UActorComponent* actorComp : this->GetComponentsByTag(UArrowComponent::StaticClass(), FName("ScopedLocationArrow")))
		{
			ScopedLocationArrow = Cast<UArrowComponent>(actorComp);
			break;
		}
		if (!IsValid(ScopedLocationArrow)) return;
		PrimaryWeapon->SetActorLocation(ScopedLocationArrow->GetComponentLocation());
		bIsScoped = true;
		PrimaryWeapon->WeaponScopedFire();
		this->GetFirstPersonCameraComponent()->SetFieldOfView(50); //change this number to a number you can change in editor eventually
	}
	if (IsValid(SecondaryWeapon) && !bIsScoped)
	{
		for (UActorComponent* actorComp : this->GetComponentsByTag(UArrowComponent::StaticClass(), FName("ScopedLocationArrow")))
		{
			ScopedLocationArrow = Cast<UArrowComponent>(actorComp);
			break;
		}
		if (!IsValid(ScopedLocationArrow)) return;
		bIsScoped = true;
		SecondaryWeapon->WeaponScopedFire();
		SecondaryWeapon->SetActorLocation(ScopedLocationArrow->GetComponentLocation());
		this->GetFirstPersonCameraComponent()->SetFieldOfView(50); //change this number to a number you can change in editor eventually
	}
}

void AEndlessVendettaCharacter::StopGunRightClick()
{
	if (IsValid(PrimaryWeapon))
	{
		bIsScoped = false;
		PrimaryWeapon->WeaponScopedFire();
		PrimaryWeapon->SetActorRelativeLocation(FVector(0,0,0));
		this->GetFirstPersonCameraComponent()->SetFieldOfView(90);
	}
	if (IsValid(SecondaryWeapon))
	{
		bIsScoped = false;
		SecondaryWeapon->WeaponScopedFire();
		SecondaryWeapon->SetActorRelativeLocation(FVector(0,0,0));
		this->GetFirstPersonCameraComponent()->SetFieldOfView(90);
	}
}

void AEndlessVendettaCharacter::GunReload()
{
	if (IsValid(PrimaryWeapon))
	{
		PrimaryWeapon->WeaponReload();
	}
	if (IsValid(SecondaryWeapon))
	{
		SecondaryWeapon->WeaponReload();
	}
}


void AEndlessVendettaCharacter::Move(const FInputActionValue& Value)
{
	// input is a Vector2D
	FVector2D MovementVector = Value.Get<FVector2D>();

	if (Controller != nullptr)
	{
		// add movement 
		AddMovementInput(GetActorForwardVector(), MovementVector.Y);
		AddMovementInput(GetActorRightVector(), MovementVector.X);
	}
}

void AEndlessVendettaCharacter::Look(const FInputActionValue& Value)
{
	// input is a Vector2D
	FVector2D LookAxisVector = Value.Get<FVector2D>();

	if (Controller != nullptr)
	{
		// add yaw and pitch input to controller
		AddControllerYawInput(LookAxisVector.X);
		AddControllerPitchInput(LookAxisVector.Y);
	}
}


void AEndlessVendettaCharacter::SetHasRifle(bool bNewHasRifle)
{
	bHasRifle = bNewHasRifle;
}

bool AEndlessVendettaCharacter::GetHasRifle()
{
	return bHasRifle;
}
